package top.daxian1218.dust.edp.component;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import top.daxian1218.dust.commons.data.HDataTable;
import top.daxian1218.dust.commons.exception.DustSystemException;
import top.daxian1218.dust.commons.exception.DustUserException;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.DictGlobalConfig;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.*;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.dict.condition.OperationType;
import top.daxian1218.dust.db.dict.condition.expressions.ColumnExpression;
import top.daxian1218.dust.db.dict.condition.expressions.ValueExpression;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.DictCodes;
import top.daxian1218.dust.edp.ui.DictComponent;
import top.daxian1218.dust.edp.ui.MenuStore;
import top.daxian1218.dust.edp.utils.SqlUtil;

import static top.daxian1218.dust.edp.utils.SqlUtil.SCRIPT_SPACE;

/**
 * @author daxian
 */
@Component
public class MenuComponent extends DictComponent {
    public static final String MENU_OBJ = "SysMenu";
    @Override
    public boolean BeforeDictObjSaveAsync(IDataAdapter adapter, MenuStore dict, DictObj obj) {
        boolean isOk = super.BeforeDictObjSaveAsync(adapter, dict, obj);
        if (!isOk) {
            return false;
        }

        if (MENU_OBJ.equals(obj.getKey())) {
            obj.forEachChanges(r ->
            {
                if (r.getState() == RowState.DETACHED || r.getState() == RowState.UNCHANGED) {
                    return;
                }

                if (r.getState() == RowState.DELETED) {
                    DeleteChildrenAsync(adapter, r.get("id").getLongValue());
                } else {
                    BindParentMenuAsync(adapter, r);
                }

            });
        }

        return true;
    }

    @Override
    public boolean BeforeDictObjSearchAsync(IDataAdapter adapter, MenuStore menuStore, DictObj obj) {
        if (MENU_OBJ.equals(obj.getKey())) {
            String parentId = menuStore.getState().get("parentId");
            if (!StringUtils.isEmpty(parentId)) {
                DictObjRow parent = this.GetMenu(adapter, TypeUtil.toLong(parentId));
                if (parent != null) {
                    DictCondition searchByParent = new DictCondition();
                    searchByParent.setOperation(OperationType.START_WITH);
                    searchByParent.setLeft(new ColumnExpression("orderLevel"));
                    searchByParent.setRight(new ValueExpression(parent.getValue("orderLevel")));
                    obj.addCondition(searchByParent);
                }
            }
        }
        return super.BeforeSearchAsync(adapter, menuStore);
    }

    private void DeleteChildrenAsync(IDataAdapter adapter, long id) {
        DictObjRow menu = GetMenu(adapter, id);
        HSqlCommand cmd = new HSqlCommand("DELETE FROM " + menu.getDictObj().getTableName() + " WHERE order_level like :orderLevel");
        cmd.setParameter("orderLevel", menu.get("orderLevel").getStringValue() + "%");
        adapter.update(cmd);
    }

    private void BindParentMenuAsync(IDataAdapter adapter, DictObjRow newMenu) {
        long parentId = newMenu.get("pid").getLongValue();
        if (parentId == 0L) {
            newMenu.setValue("orderLevel", ":" + newMenu.get("code").getStringValue());
            return;
        }

        DictObjRow parent = GetMenu(adapter, parentId);
        if (parent == null) {
            throw new BuzException("父菜单不存在");
        }

        newMenu.get("orderLevel").setValue(parent.get("orderLevel").getStringValue() + ":" + newMenu.get("code").getStringValue());
    }

    /**
     * 获取指定ID的菜单，不存在返回
     *
     * @param adapter
     * @param id
     * @return
     */
    public DictObjRow GetMenu(IDataAdapter adapter, long id) {
        DictObj objMenu = new DictObj(DictCodes.SysMenu, AppCodes.SysApp);
        objMenu.addCondition(new DictCondition("id", id));
        objMenu.search(adapter);
        return objMenu.isEmpty() ? null : objMenu.getRow(0);
    }

    @Override
    public Object DoActionAsync(IDataAdapter adapter, String actionCode, MenuStore input) {
        switch (actionCode) {
            case "orgTypes": {
                return getMenuOrgTypes(adapter, input.getState().get("id"));
            }
            case "saveOrgTypes": {
                String id = input.getState().get("id");
                JSONArray menuOrgTypes = JSONArray.parseArray(input.getState().get("menuOrgTypes"));
                saveMenuOrgTypes(adapter, id, menuOrgTypes);
                return new JSONObject();
            }
            case "dictScript":
                String id = input.getState().get("id");
                String dbType =  input.getState().get("dbType");
                if (StringUtils.isEmpty(id) || StringUtils.isEmpty(dbType)) {
                    throw new DustSystemException("字典或者数据库类型参数缺失");
                }
                switch (dbType) {
                    case "oracle":
                        return getDictScriptForOracle(adapter, id);
                }
                return StringUtils.EMPTY;
            default:
                return super.DoActionAsync(adapter, actionCode, input);
        }
    }

    private boolean saveMenuOrgTypes(IDataAdapter adapter, String id, JSONArray menuOrgTypes) {
        if (StringUtils.isEmpty(id) || menuOrgTypes == null) {
            throw new DustUserException("参数错误");
        }

        long menuId = TypeUtil.toLong(id);

        // 清空组织类型
        HSqlCommand cmdClear = new HSqlCommand("DELETE FROM SYS_MENU_ORGTYPE WHERE MENU_ID=:id");
        cmdClear.setParameter("id", menuId);
        adapter.update(cmdClear);

        // 插入组织类型
        if (menuOrgTypes.size() > 0) {
            HSqlCommand cmdAdd = new HSqlCommand("INSERT INTO SYS_MENU_ORGTYPE (ID, MENU_ID, ORG_TYPE) VALUES(:id, :menuId, :orgType)");

            menuOrgTypes.forEach( o -> {
                cmdAdd.applyBatch();
                cmdAdd.setParameter("id", DictGlobalConfig.Instance.getIdWorker().nextId());
                cmdAdd.setParameter("menuId", menuId);
                cmdAdd.setParameter("orgType", TypeUtil.toString(o));
            });
            adapter.update(cmdAdd);
        }

        adapter.commit();
        return true;
    }

    private JSONObject getMenuOrgTypes(IDataAdapter adapter, String id) {
        HSqlCommand cmd = new HSqlCommand("SELECT DUTYCODE AS Code, DUTYNAME  AS Name FROM TSYSDUTY t ");
        HDataTable dt = adapter.query(cmd);
        JSONObject result = new JSONObject();
        result.put("orgTypes", dt.toDataJSON());

        // 菜单具备组织类型
        cmd = new HSqlCommand("SELECT ORG_TYPE, MENU_ID FROM SYS_MENU_ORGTYPE WHERE MENU_ID=:id");
        cmd.setParameter("id", TypeUtil.toLong(id));
        dt = adapter.query(cmd);
        result.put("menuOrgTypes", dt.stream().map(r -> r.getString("ORG_TYPE")).toArray());
        return result;
    }

    private String getDictScriptForOracle(IDataAdapter adapter, String strId) {
        long id = TypeUtil.toLong(strId);

        DictObj sysMenu = DictObjBuilder.create(101);
        sysMenu.search(id, adapter);

        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("BEGIN").append(StringUtils.LF);

        // sys_menu
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM sys_menu WHERE id=").append(id).append(";").append(StringUtils.LF);
        SqlUtil.insertDictData(sqlBuilder, sysMenu);

        //SYS_MENU_ORGTYPE
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM SYS_MENU_ORGTYPE WHERE MENU_ID=").append(id).append(";").append(StringUtils.LF);
        HSqlCommand cmd = new HSqlCommand("SELECT ID, GMT_CREATE, GMT_MODIFIED, MENU_ID, ORG_TYPE, REMARK FROM SYS_MENU_ORGTYPE WHERE MENU_ID=:id");
        cmd.setParameter("id", TypeUtil.toLong(id));
        HDataTable dt = adapter.query(cmd);
        if (dt.size() > 0) {
            SqlUtil.insertDictData(sqlBuilder, dt, "SYS_MENU_ORGTYPE");
       }

        long uiId = sysMenu.getRows().get(0).get("uiId").getLongValue();
        if (uiId > 0) {
            sqlBuilder.append(getUIScriptForOracle(adapter, uiId));
        }

        sqlBuilder.append(SCRIPT_SPACE).append("COMMIT;").append(StringUtils.LF);
        sqlBuilder.append("END;");

        return sqlBuilder.toString();
    }

    private String getUIScriptForOracle(IDataAdapter adapter, long uiId) {

        DictObj dictUI = DictObjBuilder.create(8);
        dictUI.search(uiId, adapter);

        DictObj dictUIObj =  DictObjBuilder.create(5);
        dictUIObj.addCondition("uiId", uiId);
        dictUIObj.search(adapter);

        DictObj dictUIAction =  DictObjBuilder.create(7);
        dictUIAction.addCondition("uiId", uiId);
        dictUIAction.search(adapter);

        StringBuilder sqlBuilder = new StringBuilder();
        // dict_ui
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_ui WHERE id=").append(uiId).append(";").append(StringUtils.LF);
        SqlUtil.insertDictData(sqlBuilder, dictUI);


        // dict_ui_obj
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_ui_obj WHERE ui_id=").append(uiId).append(";").append(StringUtils.LF);
        if (!dictUIObj.isEmpty()) {
            SqlUtil.insertDictData(sqlBuilder, dictUIObj);
        }

        // dict_ui_action
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_ui_action WHERE ui_id=").append(uiId).append(";").append(StringUtils.LF);
        if (!dictUIAction.isEmpty()) {
            SqlUtil.insertDictData(sqlBuilder, dictUIAction);
        }

        return sqlBuilder.toString();
    }

}