package top.daxian1218.dust.edp.component;

import com.alibaba.fastjson.JSONArray;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import top.daxian1218.dust.commons.data.HDataRow;
import top.daxian1218.dust.commons.data.HDataTable;
import top.daxian1218.dust.commons.exception.DustSystemException;
import top.daxian1218.dust.commons.util.NameUtil;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.base.DbTypes;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.DataType;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.DictObjBuilder;
import top.daxian1218.dust.db.dict.RowState;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.ui.DictComponent;
import top.daxian1218.dust.edp.ui.MenuStore;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 字典对象组件
 * @author daxian
 */
@Component
public class ObjDictComponent extends DictComponent {
    public static final long DICT_COLUMN_ID = 822046618692878336L;
    public static final String SCRIPT_SPACE = "    ";
    public static final String QUERY_STRUCTURE_PG = "select\n" +
            "\tc.relname table_name,\n" +
            "\ta.attname physical_name,\n" +
            "\td.description column_label,\n" +
            "\tt.typname as data_type,\n" +
            "SUBSTRING (\n" +
            "\t\t\tformat_type (a.atttypid, a.atttypmod)\n" +
            "\t\t\tfrom\n" +
            "\t\t\t\t'[\\d,]+'\n" +
            "\t\t)as data_width\n" +
            "from\n" +
            "\tpg_attribute a\n" +
            "left join \tpg_class c on c.oid =a.attrelid \t\n" +
            "left join \tpg_type t on t.\"oid\" =a.atttypid\n" +
            "left join pg_description d on d.objsubid = a.attnum and d.objoid =c.\"oid\"\n" +
            "where\n" +
            "\ta.attnum > 0 and c.relname =:tableName\n" +
            "order by\n" +
            "\tc.relname,\n" +
            "\ta.attnum";

    @Override
    public boolean BeforeDictObjSaveAsync(IDataAdapter adapter, MenuStore dict, DictObj obj) {
        if (obj.getId() == DICT_COLUMN_ID) {
            List<Long> ids = obj.getChanges().stream()
                    .filter(r -> r.getState() == RowState.DELETED)
                    .map(r -> r.get("id").getLongValue())
            .collect(Collectors.toList());

            if (ids.size() > 0) {
                HSqlCommand cmd = new HSqlCommand("DELETE FROM dict_obj_column_ui WHERE column_id in (");
                List<String> placeholders = new ArrayList<>(ids.size());
                ids.forEach(id -> {
                    placeholders.add( ":" + HSqlCommand.INDEX_KEY);
                    cmd.appendParameter(id);
                });
                cmd.appendSql(String.join(",", placeholders.toArray(new String[0])));
                cmd.appendSql(")");
                adapter.update(cmd);
            }

        }
        return super.BeforeDictObjSaveAsync(adapter, dict, obj);
    }

    @Override
    public Object DoActionAsync(IDataAdapter adapter, String actionCode, MenuStore input) {
        switch (actionCode) {
            case "dictScript":
                String objId = input.getState().get("id");
                String dbType =  input.getState().get("dbType");
                if (StringUtils.isEmpty(objId) || StringUtils.isEmpty(dbType)) {
                    throw new DustSystemException("字典或者数据库类型参数缺失");
                }
                switch (dbType) {
                    case "oracle":
                        return getDictScriptForOracle(objId);
                }
                return StringUtils.EMPTY;
            case "importFromDb":
                return importFromDb(input.getState().get("tableName"), adapter);

            default:
                return super.DoActionAsync(adapter, actionCode, input);
        }

    }

    private HDataTable importFromDb(String tableName, IDataAdapter adapter) {
        if (StringUtils.isEmpty(tableName)) {
            throw new BuzException("invalid table name");
        }

        String dbType = adapter.getDbType();
        if (StringUtils.equals(dbType, DbTypes.PG)) {
            HSqlCommand queryStructure = new HSqlCommand(QUERY_STRUCTURE_PG);
            queryStructure.setParameter("tableName", tableName);
            HDataTable dt =  queryStructure.query(adapter);
            dt.forEach(r -> {
                String label = r.getString("column_label");
                if (StringUtils.isEmpty(label)) {
                    r.set("column_label", NameUtil.underscore2Camel(r.getString("physical_name")));
                } else {
                    if (label.contains("#")) {
                        r.set("column_label", label.substring(0, label.indexOf("#") ));
                    }
                }

                r.set("column_name", NameUtil.underscore2Camel(r.getString("physical_name")));
                r.set("data_type", getDataTypeFromPg(r.getString("data_type")));
                handleDataWidth(r);

            });


            return dt;
        }

        return new HDataTable();
    }

    private String getDataTypeFromPg(String pgType) {
        switch (pgType) {
            case "int4":
            case "int8":
                return DataType.UBIGINT.name();
            case "timestamp":
            case "date":
                return DataType.DATE.name();
            case "numeric":
                return DataType.NUMBER.name();
            default:
                return DataType.STRING.name();
        }
    }

    private void handleDataWidth(HDataRow row) {
        String dataWidth = row.getString("data_width");
        if (StringUtils.isEmpty(dataWidth)) {
            row.set("data_width", 0);
            row.set("data_dec", 0);
            return;
        }

        if (StringUtils.contains(dataWidth,",")) {
            String[] nums = dataWidth.split(",");
            row.set("data_width", TypeUtil.toInteger(nums[0]));
            row.set("data_dec", TypeUtil.toInteger(nums[1]));
        }
    }

    private String getDictScriptForOracle(String strObjId) {
        long objId = TypeUtil.toLong(strObjId);
        DictObj dictObj = DictObjBuilder.create(1);
        dictObj.search(objId);
        if (dictObj.isEmpty()) {
            return StringUtils.EMPTY;
        }

        DictObj dictObjColumn = DictObjBuilder.create(2);
        dictObjColumn.addCondition("objId", objId);
        dictObjColumn.search();

        DictObj dictObjTable =  DictObjBuilder.create(3);
        dictObjTable.addCondition("objId", objId);
        dictObjTable.search();

        DictObj dictObjColumnUI =  DictObjBuilder.create(821782404157931520L);
        dictObjColumnUI.addCondition("objId", objId);
        dictObjColumnUI.search();

        StringBuilder sqlBuilder = new StringBuilder();

        sqlBuilder.append("BEGIN").append(StringUtils.LF);

        // dict_obj
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_obj WHERE id=").append(objId).append(";").append(StringUtils.LF);
        insertDictData(sqlBuilder, dictObj);

        // dict_obj_column
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_obj_column WHERE obj_id=").append(objId).append(";").append(StringUtils.LF);
        if (!dictObjColumn.isEmpty()) {
            insertDictData(sqlBuilder, dictObjColumn);
        }

        // dict_obj_table
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_obj_table WHERE obj_id=").append(objId).append(";").append(StringUtils.LF);
        if (!dictObjTable.isEmpty()) {
            insertDictData(sqlBuilder, dictObjTable);
        }

        // dict_obj_column_ui
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_obj_column_ui WHERE obj_id=").append(objId).append(";").append(StringUtils.LF);
        if (!dictObjColumnUI.isEmpty()) {
            insertDictData(sqlBuilder, dictObjColumnUI);
        }

        sqlBuilder.append(SCRIPT_SPACE).append("COMMIT;").append(StringUtils.LF);
        sqlBuilder.append("END;");

        return sqlBuilder.toString();
    }

    private void insertDictData(StringBuilder sqlBuilder, DictObj dictObj) {
        StringBuilder colBuilder = new StringBuilder();
        dictObj.getColumns().forEach(col -> {
            if (colBuilder.length() > 0) {
                colBuilder.append(",");
            }

            colBuilder.append(col.getPhysicalName());
        });

        dictObj.getRows().forEach(r -> {
            StringBuilder valueBuilder = new StringBuilder();
            dictObj.getColumns().forEach(col -> {
                if (valueBuilder.length() > 0) {
                    valueBuilder.append(",");
                }
                valueBuilder.append(toOracleValue(col.getDataType(), r.get(col.getColumnName()).getValue()));
            });
            sqlBuilder.append(SCRIPT_SPACE).append("INSERT INTO ").append(dictObj.getTableName()).append(" (")
                    .append(colBuilder).append(") VALUES(").append(valueBuilder).append(");").append(StringUtils.LF);
        });
    }


    private String toOracleValue(DataType dataType, Object value) {
        switch (dataType) {
            case CURRENCY:
            case NUMBER:
            case UBIGINT:
            case INT:
                return TypeUtil.toString(value);
            case BOOLEAN:
                return TypeUtil.toBoolean(value) ? "1" : "0";
            case TEXT:
            case STRING:
            case SHORT_DATE:
            case TIME:
                return toSaveOracleString(TypeUtil.toString(value));
            case DATE:
                return "TIMESTAMP '" + TypeUtil.toString(value, "yyyy-MM-dd HH:mm:ss.SSS") + "'";
            default:
                return StringUtils.EMPTY;
        }
    }

    private String toSaveOracleString(String val) {
        if (!StringUtils.isEmpty(val)) {
            val = val.replace("'", "''");
        }

        return "'" + val + "'";
    }
}