package top.daxian1218.dust.edp.component;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import top.daxian1218.dust.commons.exception.DustSystemException;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.DataType;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.DictObjBuilder;
import top.daxian1218.dust.db.dict.RowState;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.dict.condition.OperationType;
import top.daxian1218.dust.db.dict.condition.expressions.ColumnExpression;
import top.daxian1218.dust.db.dict.condition.expressions.SubTableExpression;
import top.daxian1218.dust.edp.ui.DictComponent;
import top.daxian1218.dust.edp.ui.MenuStore;

/**
 * 界面字典服务器组件
 * @author daxian
 */
@Component
public class UIComponent extends DictComponent {
    public static final String SCRIPT_SPACE = "    ";

    @Override
    public boolean BeforeDictObjSearchAsync(IDataAdapter adapter, MenuStore menuStore, DictObj obj) {
        if (obj.getState() != null) {
            JSONObject state = (JSONObject) obj.getState();
            String menu = state.getString("menu");
            if ("*".equals(menu)) {
                DictCondition cdt = new DictCondition();
                cdt.setOperation(OperationType.NOT_EXIST);
                SubTableExpression expr = new SubTableExpression();
                expr.setTableName("sys_menu");
                expr.setColumnName("ui_id");
                expr.setAlias("menu");

                DictCondition tableCondition = new DictCondition();
                tableCondition.setLeft(new ColumnExpression("id"));
                tableCondition.setRight(new ColumnExpression("ui_id", "menu"));
                expr.getSubConditions().add(tableCondition);

                cdt.setRight(expr);

                obj.addCondition(cdt);
            }
        }
        return super.BeforeDictObjSearchAsync(adapter, menuStore, obj);
    }

    @Override
    public boolean BeforeDictObjSaveAsync(IDataAdapter adapter, MenuStore dict, DictObj obj) {
        if (!super.BeforeDictObjSaveAsync(adapter, dict, obj)) { return false; }

        if (dict.getMaster() == obj) {
            obj.forEachChanges(r -> {
                if (r.getState() == RowState.ADDED) {
                    r.setValue("id", obj.getVariableHandler().getVariable("@ID"));
                }
            });
        }

        return true;
    }

    @Override
    public Object DoActionAsync(IDataAdapter adapter, String actionCode, MenuStore input) {
        switch (actionCode) {
            case "dictScript":
                String uiId = input.getState().get("id");
                String dbType =  input.getState().get("dbType");
                if (StringUtils.isEmpty(uiId) || StringUtils.isEmpty(dbType)) {
                    throw new DustSystemException("字典或者数据库类型参数缺失");
                }
                switch (dbType) {
                    case "oracle":
                        return getDictScriptForOracle(uiId);
                }
                return StringUtils.EMPTY;
            default:
                return super.DoActionAsync(adapter, actionCode, input);
        }

    }

    private String getDictScriptForOracle(String strUiId) {
        long uiId = TypeUtil.toLong(strUiId);

        DictObj dictUI = DictObjBuilder.create(8);
        dictUI.search(uiId);

        DictObj dictUIObj =  DictObjBuilder.create(5);
        dictUIObj.addCondition("uiId", uiId);
        dictUIObj.search();

        DictObj dictUIAction =  DictObjBuilder.create(7);
        dictUIAction.addCondition("uiId", uiId);
        dictUIAction.search();

        StringBuilder sqlBuilder = new StringBuilder();

        sqlBuilder.append("BEGIN").append(StringUtils.LF);

        // dict_ui
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_ui WHERE id=").append(uiId).append(";").append(StringUtils.LF);
        insertDictData(sqlBuilder, dictUI);


        // dict_ui_obj
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_ui_obj WHERE ui_id=").append(uiId).append(";").append(StringUtils.LF);
        if (!dictUIObj.isEmpty()) {
            insertDictData(sqlBuilder, dictUIObj);
        }

        // dict_ui_action
        sqlBuilder.append(SCRIPT_SPACE).append("DELETE FROM dict_ui_action WHERE ui_id=").append(uiId).append(";").append(StringUtils.LF);
        if (!dictUIAction.isEmpty()) {
            insertDictData(sqlBuilder, dictUIAction);
        }

        sqlBuilder.append(SCRIPT_SPACE).append("COMMIT;").append(StringUtils.LF);
        sqlBuilder.append("END;");

        return sqlBuilder.toString();
    }

    private void insertDictData(StringBuilder sqlBuilder, DictObj dictObj) {
        StringBuilder colBuilder = new StringBuilder();
        dictObj.getColumns().forEach(col -> {
            if (colBuilder.length() > 0) {
                colBuilder.append(",");
            }

            colBuilder.append(col.getPhysicalName());
        });

        dictObj.getRows().forEach(r -> {
            StringBuilder valueBuilder = new StringBuilder();
            dictObj.getColumns().forEach(col -> {
                if (valueBuilder.length() > 0) {
                    valueBuilder.append(",");
                }
                valueBuilder.append(toOracleValue(col.getDataType(), r.get(col.getColumnName()).getValue()));
            });
            sqlBuilder.append(SCRIPT_SPACE).append("INSERT INTO ").append(dictObj.getTableName()).append(" (")
                    .append(colBuilder).append(") VALUES(").append(valueBuilder).append(");").append(StringUtils.LF);
        });
    }


    private String toOracleValue(DataType dataType, Object value) {
        switch (dataType) {
            case CURRENCY:
            case NUMBER:
            case UBIGINT:
            case INT:
                return TypeUtil.toString(value);
            case BOOLEAN:
                return TypeUtil.toBoolean(value) ? "1" : "0";
            case TEXT:
            case STRING:
            case SHORT_DATE:
            case TIME:
                return toSaveOracleString(TypeUtil.toString(value));
            case DATE:
                return "TIMESTAMP '" + TypeUtil.toString(value, "yyyy-MM-dd HH:mm:ss.SSS") + "'";
            default:
                return StringUtils.EMPTY;
        }
    }

    private String toSaveOracleString(String val) {
        if (!StringUtils.isEmpty(val)) {
            val = val.replace("'", "''");
        }

        return "'" + val + "'";
    }
}
