package top.daxian1218.dust.edp.services;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import top.daxian1218.dust.commons.log.LoggerHelper;
import top.daxian1218.dust.commons.thread.LocalHolder;
import top.daxian1218.dust.commons.util.BeanUtil;
import top.daxian1218.dust.db.DictGlobalConfig;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.EdpProperties;
import top.daxian1218.dust.edp.services.queryProject.AdvancedQueryService;
import top.daxian1218.dust.edp.services.storage.StorageService;
import top.daxian1218.dust.edp.services.sys.MyService;
import top.daxian1218.dust.edp.ui.*;
import top.daxian1218.dust.edp.utils.JsonUtil;
import top.daxian1218.dust.service.security.SecurityUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * @author daxian
 */
@Component
public class UIService extends BaseService {
    private final DictLoader dictLoader;
    private final BuzServiceManager buzServiceManager;
    private final RightService rightService;
    private final EdpProperties edpProperties;
    private final StorageService storageService;
    private final AdvancedQueryService advancedQueryService;
    private final MyService myService;

    public UIService(StorageService storageService,
                     DictLoader dictLoader,
                     AdvancedQueryService advancedQueryService,
                     BuzServiceManager buzServiceManager,
                     RightService rightService,
                     MyService myService,
                     EdpProperties edpProperties) {
        this.dictLoader = dictLoader;
        this.buzServiceManager = buzServiceManager;
        this.rightService = rightService;
        this.edpProperties = edpProperties;
        this.storageService = storageService;
        this.advancedQueryService = advancedQueryService;
        this.myService = myService;
    }

    public Object dispatchAsync(String method,
                                JSONObject input,
                                HttpServletRequest request,
                                HttpServletResponse response) {
        if (StringUtils.isEmpty(method)) {
            throw new BuzException("method not found");
        }
        input = Optional.ofNullable(input).orElse(new JSONObject());

        switch (method) {
            case "doAction":
                String actionCode = input.getString("action");
                return parseMenuStore(input).doActionAsync(actionCode);
            case "search":
                return JsonUtil.toSearchObjSchemas(parseMenuStore(input).search());
            case "save":
                return JsonUtil.toSaveObjSchemas(parseMenuStore(input).save());
            case "getUI": {
                return getUI(input.getString("menuId"));
            }
            case "getData":
                return GetDataDictAsync(input.getString("codes"), input.getString("categories"));
            case "getObjByCode":
                return GetObjDictAsync(input.getString("codes"), input.getString("appCode"));
            case "getObjById":
                JSONArray strIds = input.getJSONArray("ids");
                long[] ids = new long[strIds.size()];
                for (int i = 0; i < strIds.size(); i++) {
                    ids[i] = strIds.getLongValue(i);
                }
                return GetObjDictAsync(ids);
            case "downloadFile":
                downloadFile(input, response);
                return true;
            case "uploadFile":
                return this.uploadFile(request);
            case "queryProject":
                return this.queryProject(input);
            default:
                return buzServiceManager.ExecutePluginAsync(method, DictGlobalConfig.Instance.getDataAdapter(), input);
        }
    }

    private Object queryProject(JSONObject input) {
        String action = input.getString("action");
        switch (action) {
            case "load":
                return advancedQueryService.listProjectItems(input.getString("menuId"), input.getString("objId"));
            case "project":
                return advancedQueryService.getQueryProject(input.getString("menuId"), input.getString("objId"));
            case "update":
                return advancedQueryService.updateProject(input);
            default:
                throw new BuzException("查询方案无效操作");
        }
    }

    private void downloadFile(JSONObject input, HttpServletResponse resp) {
        if (!input.containsKey("id")) {
            throw new BuzException("文件ID无效");
        }
        this.storageService.download(input.getString("id"), resp);
    }

    private  String uploadFile(HttpServletRequest req) {
        List<String> fileIds = this.storageService.uploadFiles(req);
        if (fileIds.size() > 0) {
            return fileIds.get(0);
        }

        throw new BuzException("文件上传失败，无法获取上传文件的ID");
    }

    public MenuStore parseMenuStore(String menuId) {
        JSONObject input = new JSONObject();
        input.put("menuId", menuId);
        return parseMenuStore(input);
    }

    public MenuStore parseMenuStore(JSONObject inParams) {
        // 不依赖菜单的操作
        // TODO 重构，读取缓存
        String menuId = inParams.getString("menuId");
        MenuStore store;
        if (!StringUtils.isEmpty(menuId) && !StringUtils.equals(menuId, "*")) {
            LocalHolder.set("menuId", menuId);
            store = new MenuStore(menuId);
            JSONObject menuDict = dictLoader.getMenuDict(menuId);
            if (!menuDict.isEmpty()) {
                store.setTitle(menuDict.getString("title"));
                JSONObject uiDict = menuDict.getJSONObject("uiDict");
                if (uiDict != null) {
                    uiDict.put("objs", menuDict.get("objs"));
                    uiDict.put("actions", menuDict.get("actions"));
                    store.setUiDict(uiDict.toJavaObject(UIDict.class));

                    String backendComponent = store.getUiDict().getBackendComponent();
                    if (!StringUtils.isEmpty(backendComponent)) {
                        IBiz biz = (IBiz) BeanUtil.getBean(backendComponent);
                        store.setBiz(biz);
                    }
                }
            }
        } else {
            store = new MenuStore("*");
        }

        store.setAdapter(getDefaultAdapter());

        // 附加状态信息
        JSONObject state = inParams.getJSONObject("state");
        if (state != null) {
            for (Map.Entry<String, Object> kv : state.entrySet()) {
                store.getState().put(kv.getKey(), state.getString(kv.getKey()));
            }
        }

        if (store.getBiz() == null) {
            store.setBiz(getDefaultComponent());
        }

        // 加载前端信息
        store.loadJSONAsync(inParams.getJSONArray("objs"));
        return store;
    }

    private DictComponent getDefaultComponent() {
        if (!StringUtils.isEmpty(this.edpProperties.getDefaultComponent())) {
            Object bean = BeanUtil.getBean(this.edpProperties.getDefaultComponent());
            if (bean instanceof DictComponent) {
                return (DictComponent) bean;
            }

            LoggerHelper.error("EDP配置无效，无法加载对应的默认组件，启用系统默认组件");
        }

        Object bean = BeanUtil.getBean("defaultComponent");
        if (bean instanceof DictComponent) {
            return (DictComponent) bean;
        }

        return new DefaultComponent();
    }

    public JSONArray GetDataDictAsync(String codes, String categories) {
        return dictLoader.getDataDict(codes, categories);
    }

    public JSONObject getUI(String menuId) {
        if (StringUtils.isEmpty(menuId)) {
            return new JSONObject();
        }

        JSONObject menuDict = dictLoader.getMenuDict(menuId);
        menuDict.remove("uiDict");
        menuDict.put("profile", myService.getMenuProfile(menuId));
        return filterAction(menuDict, menuId);
    }

    private JSONObject filterAction(JSONObject jsonUI, String menuId) {
        if (SecurityUtils.isSuperAdmin()) {
            return jsonUI;
        }
        List<String> roleActions = rightService.getAction(menuId);
        JSONArray arr = jsonUI.getJSONArray("actions");
        filterAction(arr, roleActions);
        return  jsonUI;
    }

    private void filterAction(JSONArray children, List<String> roleActions) {
        for (int i = children.size() - 1; i >= 0; i--) {
            JSONObject item = children.getJSONObject(i);
            if (!roleActions.contains(item.getString("id"))) {
                children.remove(item);
                continue;
            }

            if (item.containsKey("children")) {
                filterAction(item.getJSONArray("children"), roleActions);
            }
        }
    }

    public JSONObject GetObjDictAsync(String codes, String appCode) {
        return dictLoader.getObjDictAsync(codes, appCode);
    }

    public JSONObject GetObjDictAsync(long[] ids) {
        return dictLoader.getObjDictAsync(ids);
    }

}