package top.daxian1218.dust.edp.services.authentication;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import top.daxian1218.dust.commons.util.PasswordUtil;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.DictGlobalConfig;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.DictObjBuilder;
import top.daxian1218.dust.db.dict.DictObjRow;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.ErrorCodes;
import top.daxian1218.dust.service.security.DustUser;
import top.daxian1218.dust.service.security.SecurityUtils;
import top.daxian1218.dust.service.security.jwt.TokenProvider;

import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.util.stream.Collectors;

public class UserService implements IUserService {
    private final Logger logger = LoggerFactory.getLogger(UserService.class);

    private final long USER_ID = 10008L;
    private final long ROLE_ID = 10011L;
    private final long RIGHT_ID = 10002L;
    private final long TOKEN_ID = 10014L;
    private final TokenProvider tokenProvider;

    public UserService(TokenProvider tokenProvider) {
        this.tokenProvider = tokenProvider;
    }

    @Override
    public JSONObject login(JSONObject input) {
        String userCode = input.getString("user");
        String password = input.getString("password");
        DictObj objUser = DictObjBuilder.create(USER_ID);
        objUser.addCondition(new DictCondition("code", userCode));
        DictCondition mobileCdt = new DictCondition("mobile", userCode);
        mobileCdt.setRequire((false));
        objUser.addCondition(mobileCdt);
        objUser.search();

        if (objUser.isEmpty() || objUser.getRow(0).get("status").getInt32Value() != 1) {
            throw new BuzException("用户不存在", ErrorCodes.USER_MISS);
        }

        DictObjRow curUser = objUser.getRow(0);
        if (!validatePassword(curUser.get("code").getStringValue(), curUser.get("password").getStringValue(), password))
        {
            throw new BuzException("账户或者密码错误", ErrorCodes.PASSWORD_ERROR);
        }

        DustUser loginUser = new DustUser();
        loginUser.setSid(TypeUtil.toString(DictGlobalConfig.Instance.getIdWorker().nextId()));
        loginUser.setId(curUser.get("id").getStringValue());
        loginUser.setCode(curUser.get("code").getStringValue());
        loginUser.setName(curUser.get("name").getStringValue());

        SecurityUtils.setUser(loginUser);

        JSONObject user = getUser();

        loginUser.setRoles(user.getString("roles"));
        loginUser.setRights(user.getString("rights"));
        loginUser.setProps(StringUtils.EMPTY);
        updateUserContextToken();

        user.put("token", loginUser.getToken());
        user.put("refreshToken", loginUser.getRefreshToken());
        return user;
    }

    private boolean validatePassword(String code, String password, String input)  {
        try {
            return  StringUtils.equals(password,PasswordUtil.encryptByPdkdf2(code.toUpperCase() + input));
        } catch (InvalidKeySpecException | NoSuchAlgorithmException e ) {
            throw new BuzException("无效的加密算法");
        }
    }

    private void updateUserContextToken()
    {
        DustUser user = SecurityUtils.getUserInfo();
        user.setToken(tokenProvider.createJwtToken(user, false));
        user.setRefreshToken(tokenProvider.createRefreshToken(user));

        // TODO 写入缓存
        DictObj obj = DictObjBuilder.create(TOKEN_ID);
        DictObjRow newRow = obj.addRow();
        newRow.get("id").setValue(user.getSid());
        newRow.get("accessToken").setValue(user.getToken());
        newRow.get("refreshToken").setValue(user.getRefreshToken());
        obj.save();
    }

    public void logout()
    {
        //TODO 失效当前Token

        return;
    }

    @Override
    public JSONObject refreshToken()
    {
        String sid = SecurityUtils.getUserInfo().getSid();
        DictObj objToken = DictObjBuilder.create(TOKEN_ID);
        objToken.search(TypeUtil.toLong(sid));
        if (objToken.isEmpty())
        {
            throw new BuzException("刷新Token已过期或失效");
        }

        SecurityUtils.setUser(tokenProvider.convertToUser(objToken.getRow(0).get("token").getStringValue()));
        updateUserContextToken();

        JSONObject result =  new JSONObject();
        result.put("token", SecurityUtils.getUserInfo().getToken());
        result.put("refreshToken", SecurityUtils.getUserInfo().getRefreshToken());
        return result;
    }

    public JSONObject getUser()
    {
        long id = TypeUtil.toLong(SecurityUtils.getUserInfo().getId());
        DictObj sysUser = DictObjBuilder.create(USER_ID);
        sysUser.search(id);
        if (sysUser.isEmpty())
        {
            return null;
        }

        JSONObject user = sysUser.getRow(0).toJSON();
        user.put("roles", getRoles(id));
        user.put("rights", getRights(id));
        user.remove("password");
        return user;
    }

    private String getRoles(long id)
    {
        DictObj objUserRole = DictObjBuilder.create(ROLE_ID);
        objUserRole.addCondition(new DictCondition("userId", id));
        objUserRole.search();

        return objUserRole.getRows().stream().map(r -> r.get("roleCode").getStringValue()).collect(Collectors.joining(","));
    }

    private String getRights(long id)
    {
        DictObj objUserRight = DictObjBuilder.create(RIGHT_ID);
        objUserRight.addCondition(new DictCondition("userId", id));
        objUserRight.search();
        return objUserRight.getRows().stream().map(r -> r.get("userRight").getStringValue()).collect(Collectors.joining(","));
    }

}