package top.daxian1218.dust.edp.services.dev;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.dict.*;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.services.BaseService;
import top.daxian1218.dust.edp.utils.JsonUtil;

import java.util.HashMap;
import java.util.Map;

public class MenuMangerService extends BaseService {
    private Logger _logger = LoggerFactory.getLogger(MenuMangerService.class);

    public MenuMangerService() {

    }

    private final String DICT_MENU = "DictMenu";

    /**
     * 加载父菜单的子菜单
     *
     * @param parentId
     */
    public JSONArray LoadMenus(String parentId) {
        if (StringUtils.isEmpty(parentId)) {
            parentId = "0";
        }

        DictObj objMenu = new DictObj(DICT_MENU, AppCodes.DictApp);
        objMenu.addCondition(new DictCondition("parentId", parentId));

        OrderBy numOrder = new OrderBy();
        numOrder.setColumnName("orderNum");
        numOrder.setOrderByType(OrderByType.ASC);
        objMenu.addOrderBy(numOrder);

        objMenu.search();
        JSONArray result = new JSONArray();
        objMenu.getRows().forEach(r ->
        {
            JSONObject jo = r.toJSON();
            result.add(jo);
        });

        return result;
    }

    /**
     * 获取指定ID的菜单，不存在返回null
     *
     * @param id
     */
    public DictObjRow GetMenu(long id) {
        DictObj objMenu = new DictObj(DICT_MENU, AppCodes.DictApp);
        objMenu.addCondition(new DictCondition("id", id));
        objMenu.search();
        return objMenu.getFirstOrDefaultRow();
    }

    /**
     * 创建菜单，有且仅有一新增行时，才会创建。同时返回新增的菜单。否则为 null
     *
     * @param body
     */
    public DictObjRow CreateMenu(JSONObject body) {
        DictObj objMenu = new DictObj(DICT_MENU, AppCodes.DictApp);
        JsonUtil.JsonToRow(body, objMenu);
        if (objMenu.getRows().size() == 1 && objMenu.getRow(0).getState() == RowState.ADDED) {
            DictObjRow newMenu = objMenu.getRow(0);
            BindParentMenuAsync(newMenu);
            objMenu.save(getDefaultAdapter());
            getDefaultAdapter().commit();
            return GetMenu(objMenu.getRow(0).get("id").getLongValue());
        }

        return null;
    }

    private void BindParentMenuAsync(DictObjRow newMenu) {
        long parentId = newMenu.get("parentId").getLongValue();
        if (parentId == 0L) {
            newMenu.get("orderLevel").setValue(newMenu.get("code").getValue() + "#");
        } else {
            DictObjRow parent = GetMenu(parentId);
            if (parent == null) {
                throw new BuzException("父菜单不存在");
            }
            newMenu.get("orderLevel").setValue(parent.get("orderLevel").getStringValue() + newMenu.get("code").getStringValue() + "#");
        }

        // 更新菜单顺序
        int orderNum = TypeUtil.toInteger(newMenu.get("orderNum").getValue());
        if (orderNum <= 0) {
            orderNum = 1;
            newMenu.get("orderNum").setValue(orderNum);
        }

        DictObj objMenu = new DictObj(DICT_MENU, AppCodes.DictApp);
        objMenu.addCondition(new DictCondition("parentId", parentId));
        objMenu.search();
        int finalOrderNum = orderNum;
        objMenu.getRows().forEach(row ->
        {
            if (row.get("id") == newMenu.get("id").getValue()) return;

            int otherOrder = TypeUtil.toInteger(row.get("orderNum").getValue());
            if (otherOrder >= finalOrderNum) {
                row.get("orderNum").setValue(otherOrder + 1);
            }
        });

        objMenu.save(getDefaultAdapter());


    }

    /**
     * 更新菜单，有且仅有一行编辑行时，才会更新。同时返回编辑后的菜单。否则为 null
     *
     * @param body
     * @return
     */
    public DictObjRow UpdateMenu(JSONObject body) {
        DictObj objMenu = new DictObj(DICT_MENU, AppCodes.DictApp);
        body.put("$rowState", RowState.MODIFIED.toString());
        JsonUtil.JsonToRow(body, objMenu);
        if (objMenu.getRows().size() == 1 && objMenu.getRow(0).getState() == RowState.MODIFIED) {
            BindParentMenuAsync(objMenu.getRow(0));
            objMenu.save(getDefaultAdapter());
            getDefaultAdapter().commit();
            return GetMenu(objMenu.getRow(0).get("id").getLongValue());
        }

        return null;
    }

    /**
     * 删除菜单，如果不存在的菜单，仍然认为删除成功
     *
     * @param id
     */
    public boolean DeleteMenu(long id) {
        DictObjRow row = GetMenu(id);
        if (row == null) {
            return true;
        }

        HSqlCommand cmd = new HSqlCommand("DELETE FROM " + row.getDictObj().getTableName() + " WHERE order_level like :orderLevel");
        cmd.setParameter("orderLevel", row.get(" orderLevel").getStringValue() + "%");
        getDefaultAdapter().update(cmd);
        getDefaultAdapter().commit();
        return true;
    }

    public Object GetMenuTree() {
        DictObj objMenu = new DictObj(DICT_MENU, AppCodes.DictApp);

        OrderBy pOrder = new OrderBy();
        pOrder.setColumnName("parentId");
        pOrder.setOrderByType(OrderByType.ASC);
        objMenu.addOrderBy(pOrder);

        OrderBy numOrder = new OrderBy();
        numOrder.setColumnName("orderNum");
        numOrder.setOrderByType(OrderByType.ASC);
        objMenu.addOrderBy(numOrder);

        objMenu.search();

        JSONArray menuTree = new JSONArray();
        Map<Long, JSONArray> pIdMap = new HashMap<>();
        objMenu.getRows().forEach(r ->
        {
            long id = r.get("id").getLongValue();
            long pId = r.get("parentId").getLongValue();
            JSONObject node = r.toJSON();
            if (pId == 0) {
                menuTree.add(node);
            } else {
                if (!pIdMap.containsKey(pId)) {
                    pIdMap.put(pId, new JSONArray());
                }
                pIdMap.get(pId).add(node);
            }
        });

        for (Object item : menuTree) {
            JSONObject jo = (JSONObject) item;
            jo.put("children", FindChildren(jo.getLong("id"), pIdMap));
        }

        return menuTree;
    }

    private JSONArray FindChildren(long pId, Map<Long, JSONArray> src) {
        if (src.containsKey(pId)) {
            JSONArray children = new JSONArray(src.get(pId));
            for (Object item : children) {
                JSONObject jo = (JSONObject) item;
                jo.put("children", FindChildren(jo.getLong("id"), src));
            }

            return children;
        }
        return new JSONArray();
    }
}