package top.daxian1218.dust.edp.services.io;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.DictObjBuilder;
import top.daxian1218.dust.db.dict.DictObjColumn;
import top.daxian1218.dust.db.dict.DictObjRow;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.services.BuzService;
import top.daxian1218.dust.edp.services.UIService;
import top.daxian1218.dust.edp.services.storage.StorageService;
import top.daxian1218.dust.edp.ui.DictLoader;
import top.daxian1218.dust.edp.ui.MenuStore;
import top.daxian1218.dust.edp.ui.UIDict;
import top.daxian1218.dust.edp.utils.ExcelUtil;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.text.NumberFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author daxian
 */
@Service
public class ExcelService extends BuzService {
    Logger logger = LoggerFactory.getLogger(ExcelService.class);
    public static final String IO_TYPE = "ioType";
    public static final String IO_TYPE_IM = "1";
    public static final String IO_TYPE_OUT = "0";
    private static final NumberFormat NUMBER_FORMAT = NumberFormat.getNumberInstance();
    private final StorageService storageService;
    private final UIService uiService;
    private final DictLoader _dictLoader;

    static {
        NUMBER_FORMAT.setGroupingUsed(false);
    }


    public ExcelService(UIService uiService, StorageService storageService, DictLoader dictLoader) {
        this.uiService = uiService;
        this.storageService = storageService;
        _dictLoader = dictLoader;
    }

    @Override
    public Object InvokeAsync(JSONObject inParams) {
        if (!inParams.containsKey(IO_TYPE)) {
            return null;
        }

        String ioType = inParams.getString(IO_TYPE);
        if (StringUtils.equals(ioType, IO_TYPE_IM)) {
            return importData(inParams.getJSONObject("data"));
        }

        if (StringUtils.equals(ioType, IO_TYPE_OUT)) {
            exportData(inParams.getJSONObject("data"));
            return StringUtils.EMPTY;
        }

        throw new BuzException("无效的Excel操作");
    }

    private ImportResult importData(JSONObject inParams) {
        ImportResult result = new ImportResult();
        if (!inParams.containsKey("fileId")) {
            throw new BuzException("参数无效");
        }

        String fileId = inParams.getString("fileId");
        InputStream fileStream = storageService.getFile(fileId);
        if (fileStream == null) {
            throw new BuzException("无法找到数据文件");
        }
        try {
            storageService.delete(fileId);
        } catch (Exception ex) {
            logger.error("Excel导入，删除临时文件失败-" + fileId, ex);
        }

        String menuId = inParams.getString("menuId");
        if (StringUtils.isEmpty(menuId)) {
            throw new BuzException("菜单参数无效");
        }

        MenuStore store = uiService.parseMenuStore(inParams);
        if (store.getObjList().size() <= 0) {
            throw new BuzException("导入对象无效");
        }

        try {
            Workbook wk = XSSFWorkbookFactory.create(fileStream);
            Sheet sheet = wk.getSheetAt(0);
            if (sheet.getLastRowNum() <= 0) {
                result.setRowCount(0);
                return result;
            }

            // 加载数据到对象字典
            DictObj impObj = store.getObjList().get(0);

            Row rowCol = sheet.getRow(0);
            Map<String, Short> columnIndexes = getExcelColumns(impObj, rowCol);


            if (StringUtils.isEmpty(impObj.getTableName())) {
                throw new BuzException("无效的对象字典");
            }

            loadData(sheet, columnIndexes, impObj);

            // 触发自定义处理逻辑
            ArrayList<DictObj> objList = new ArrayList<>(1);
            objList.add(impObj);

            store.getBiz().ImportData(getDataAdapter(), store, objList);

            int len = impObj.getRows().size();
            for (int i = len - 1; i >= 0; i--) {
                DictObjRow r = impObj.getRow(i);
                if (!r.validate()) {
                    ImportResult.RowError rowError = new ImportResult.RowError();
                    rowError.setError(getRowErrors(r));
                    rowError.setRowNum(i + 1);
                    result.getErrors().add(rowError);
                    impObj.removeRow(r);
                }
            }

            IDataAdapter adapter = getDataAdapter();
            impObj.save();
            adapter.commit();

            result.setRowCount(impObj.getRows().size());
            result.setAllPassed(result.getErrors().size() == 0);
            return result;

        } catch (IOException e) {
            logger.error("打开导入文件失败", e);
            throw new BuzException("打开导入文件失败:" + e.getMessage());
        }
    }

    private String getRowErrors(DictObjRow row) {
        StringBuilder sBuilder = new StringBuilder();
        row.getErrors().forEach(err -> {
            if (sBuilder.length() > 0) {
                sBuilder.append("\n");
            }
            sBuilder.append("[");
            sBuilder.append(row.getDictObj().getColumn(err.getColumnName()).getColumnLabel());
            sBuilder.append("]");

            sBuilder.append("" + err.getMessage());
        });

        return sBuilder.toString();
    }

    private Map<String, Short> getExcelColumns(DictObj impObj, Row rowCol) {
        short minColIx = rowCol.getFirstCellNum();
        short maxColIx = rowCol.getLastCellNum();
        Map<String, String> columnMap = new HashMap<>(impObj.getColumns().size());
        impObj.getColumns().forEach(col -> {
            columnMap.put(col.getColumnLabel(), col.getColumnName());
        });

        Map<String, Short> columnIndexes = new HashMap<>(maxColIx);
        for (short colIx = minColIx; colIx < maxColIx; colIx++) {
            Cell cell = rowCol.getCell(colIx);
            if (cell == null) {
                continue;
            }

            if (cell.getCellType() == CellType.NUMERIC || cell.getCellType() == CellType.FORMULA) {
                continue;
            }

            String dispName = cell.getStringCellValue();
            if (!columnMap.containsKey(dispName)) {
                continue;
            }

            columnIndexes.put(columnMap.get(dispName), colIx);
        }
        return columnIndexes;
    }

    private void loadData(Sheet sheet, Map<String, Short> columnIndexes, DictObj impObj) {
        int maxRowIx = sheet.getLastRowNum();
        for (int rowIx = 1; rowIx <= maxRowIx; rowIx++) {
            DictObjRow newRow = impObj.addRow();
            Row dataRow = sheet.getRow(rowIx);
            for (String key : columnIndexes.keySet()) {
                short colIx = columnIndexes.get(key);
                Cell cell = dataRow.getCell(colIx);
                if (cell == null) {
                    newRow.setValue(key, null);
                    continue;
                }

                switch (cell.getCellType()) {
                    case STRING:
                        newRow.setValue(key, cell.getStringCellValue());
                        break;
                    case NUMERIC:
                        double d = cell.getNumericCellValue();
                        newRow.setValue(key, NUMBER_FORMAT.format(d));
                        break;
                    case BOOLEAN:
                        newRow.setValue(key, cell.getBooleanCellValue());
                        break;
                    case FORMULA:
                        newRow.setValue(key, ExcelUtil.getFormulaResult(cell));
                        break;
                    default:
                        newRow.setValue(key, null);
                }

            }
        }
    }

    private void exportData(JSONObject inParams) {

        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (servletRequestAttributes == null) {
            throw new BuzException("invalid request");
        }

        HttpServletResponse response = servletRequestAttributes.getResponse();

        MenuStore store = uiService.parseMenuStore(inParams);
        store.getObjList().forEach(obj -> {
            obj.getPageInfo().setPageSize(0);
        });
        store.search();

        List<DictObj> objList = store.getObjList();
        store.getBiz().ExportData(getDataAdapter(), store, objList);
        SXSSFWorkbook workbook = new SXSSFWorkbook (100);
        if (objList.size() == 0) {
            workbook.createSheet().createRow(0).createCell(0).setCellValue("无数据");
        }
        objList.forEach(dictObj -> {
            Sheet  sheet = workbook.createSheet(dictObj.getName());
            Row columnRow = sheet.createRow(0);

            List<DictObjColumn> columns = dictObj.getColumns().toList().stream()
                    .filter(c -> c.getExtendedProperties().getInt("dispPosition") <= 1)
                    .sorted(Comparator.comparingInt(a -> a.getExtendedProperties().getInt("dispIndex")))
                    .collect(Collectors.toList());
            int colLen = columns.size();
            for (int i = 0; i < colLen; i++) {
                Cell cell = columnRow.createCell(i, CellType.STRING);
                cell.setCellValue(columns.get(i).getColumnLabel());
            }

            int rowLen = dictObj.getRows().size();
            for (int i = 0; i < rowLen; i++) {
                Row row = sheet.createRow(i + 1);
                DictObjRow data = dictObj.getRow(i);
                for (int j = 0; j < columns.size(); j++) {
                    Cell cell = row.createCell(j, CellType.STRING);
                    cell.setCellValue(data.getText(columns.get(j).getColumnName()));
                }
            }
        });

        try {
            workbook.write(response.getOutputStream());
            response.addHeader(HttpHeaders.CONTENT_TYPE, "application/octet-stream");
            response.addHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=" + store.getTitle() + ".xlsx");
            response.getOutputStream().flush();
            response.getOutputStream().close();
        } catch (IOException e) {
            throw new BuzException("导出数据出错:" + e.getMessage(), null, e);
        }
    }
}
