package top.daxian1218.dust.edp.services.storage;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.system.ApplicationHome;
import org.springframework.stereotype.Component;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.commons.id.SnowFlakeIdWorker;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.DictObjRow;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.BuzException;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * @author daxian
 */
@Component
public class LocalFileRepository implements IFileRepository {
    public static final Logger logger = LoggerFactory.getLogger(LocalFileRepository.class);
    private static final int MAX_SEQ = 1000;
    public static final String DICT_FILE = "SysFile";

    private final SnowFlakeIdWorker idWorker = new SnowFlakeIdWorker(2, 0);

    private boolean override = false;
    private StorageOptions storageOptions;
    private File rootDirectory;
    private int seqNo = 0;

    public LocalFileRepository(StorageOptions options) {
        storageOptions = options;

        File file = new File(getRootPath());
        if (file.isFile()) {
            throw new BuzException("路径非法，配置路径为文件");
        }

        if (!file.exists() && !file.mkdirs()) {
            throw new BuzException("无法创建对应的文件存储路径：" + file.getAbsolutePath());
        }

        this.rootDirectory = file;
    }

    private String getRootPath() {
        String path = storageOptions.getLocal().getFilePath();
        if (StringUtils.isEmpty(path)) {
            ApplicationHome home = new ApplicationHome(getClass());
            path = home.getSource().getParentFile().getAbsolutePath();
        }

        if (StringUtils.isEmpty(path)) {
            path = System.getProperty("user.dir");
        }

        try {
            path = path.replace("/", File.separator).replace("\\", File.separator);
            return new File(path).getCanonicalPath();
        } catch (IOException e) {
            return new File(path).getAbsolutePath();
        }
    }

    private synchronized String getNewFileName() {
        if (seqNo > MAX_SEQ) {
            seqNo = 0;
        }

        return TypeUtil.toStringOfDateTime(new Date(), "yyyyMMddHHmmss") + String.format("%04d", ++seqNo);
    }

    @Override
    public FormData upload(HttpServletRequest request) {
        if (request == null) {
            throw new BuzException("request is null");
        }
        MultipartHttpServletRequest multipartRequest = null;
        if (request instanceof MultipartHttpServletRequest) {
            multipartRequest = (MultipartHttpServletRequest) request;
        }

        List<FileInfo> files = multipartRequest == null ? uploadFileByStream(request) : uploadFileByFormData(multipartRequest);
        if (logger.isInfoEnabled()) {
            logger.info("files has upload:" + files.stream().map(FileInfo::toString).reduce(String::concat));
        }

        DictObj obj = new DictObj(DICT_FILE, "sys");

        files.forEach(fileInfo -> {
            DictObjRow row = obj.addRow();
            row.get("id").setValue(fileInfo.getFileId());
            row.get("fileName").setValue(fileInfo.getFileName());
            row.get("fullName").setValue(fileInfo.getFilePath());
            row.get("filePath").setValue(fileInfo.getFilePath());
        });

        FormData formData = new FormData();
        formData.setFiles(files);

        multipartRequest.getParameterMap().forEach((key, values) -> {
            formData.set(key, values != null && values.length > 0 ? values[0] : null);
        });

        if (obj.getRows().size() > 0) {
            obj.save();
        }
        return formData;
    }

    private List<FileInfo> uploadFileByFormData(MultipartHttpServletRequest request) {
        List<FileInfo> files = new ArrayList<>();
        request.getFileMap().forEach((name, multiFile) -> {
            if (multiFile.getSize() > 0) {
                FileInfo file = new FileInfo();
                file.setFileId(String.valueOf(idWorker.nextId()));

                String fileName = multiFile.getOriginalFilename();
                if (StringUtils.isEmpty(fileName)) {
                    fileName = multiFile.getName();
                }

                if (StringUtils.isEmpty(fileName)) {
                    fileName = file.getFileId();
                }

                file.setFileName(fileName);
                try {
                    file.setFilePath(saveFileToLocal(file.getFileName(), multiFile.getInputStream()));
                } catch (IOException e) {
                    throw new BuzException("请求流读取失败");
                }

                files.add(file);
            }
        });

        return files;
    }

    private List<FileInfo> uploadFileByStream(HttpServletRequest request) {
        FileInfo fileInfo = new FileInfo();
        fileInfo.setFileId(String.valueOf(idWorker.nextId()));

        String fileName = request.getParameter("name");
        if (StringUtils.isEmpty(fileName)) {
            fileName = fileInfo.getFileId();
        }
        fileInfo.setFileName(fileName);

        try {
            String path = saveFileToLocal(fileName, request.getInputStream());
            fileInfo.setFilePath(path);
        } catch (IOException e) {
            throw new BuzException("请求流读取失败");
        }

        return Collections.singletonList(fileInfo);
    }

    @Override
    public void download(String id, OutputStream os) {
        Preconditions.checkNotEmpty(id);

        DictObj obj = new DictObj(DICT_FILE, AppCodes.SysApp);
        obj.addCondition(new DictCondition("id", id));
        obj.search();
        if (obj.getRows().size() == 0) {
            throw new BuzException("file id is invalid", "0007");
        }

        String filePath = obj.getRow(0).getStringValue("filePath");
        downloadFile(filePath, os);
    }

    @Override
    public void delete(String id) {
        Preconditions.checkNotEmpty(id);

        DictObj obj = new DictObj(DICT_FILE, AppCodes.SysApp);
        obj.addCondition(new DictCondition("id", id));
        obj.search();
        if (obj.getRows().size() == 0) {
            throw new BuzException("file id is invalid", "0007");
        }

        String filePath = obj.getRow(0).getStringValue("filePath");
        deleteFile(filePath);
        logger.info("file(" + filePath + ") had be deleted");
    }

    @Override
    public boolean exists(String id) {
        return exists(id);
    }

    @Override
    public String uploadFile(String filePath, InputStream source) {
        String path = saveFileToLocal(filePath, source);
        DictObj obj = new DictObj(DICT_FILE, "sys");
        DictObjRow row = obj.addRow();
        row.setValue("id", String.valueOf(idWorker.nextId()));
        row.setValue("fileName", filePath);
        row.setValue("fullName", path);
        row.setValue("filePath", path);
        obj.save();
        return row.getStringValue("id");
    }

    public String saveFileToLocal(String filePath, InputStream source) {
        if (StringUtils.isEmpty(filePath)) {
            throw new BuzException("file name is empty");
        }

        File file = new File(rootDirectory, filePath);
        if (file.isDirectory()) {
            file = new File(file.getAbsolutePath(), getNewFileName());
        }

        if (!override && file.exists()) {
            file = new File(file.getParent(), getNewFileName());
        }

        try {
            FileOutputStream fos = new FileOutputStream(file);
            FileCopyUtils.copy(source, fos);
        } catch (IOException e) {
            logger.error("文件上传失败", e.getMessage());

            return null;
        }

        return file.getAbsolutePath();
    }

    @Override
    public void downloadFile(String filePath, OutputStream destination) {
        if (StringUtils.isEmpty(filePath)) {
            throw new BuzException("file name is empty");
        }

        String dir = rootDirectory.getAbsolutePath();
        if (!filePath.contains(dir)) {
            BuzException buzException = new BuzException("文件地址权限不足，无法访问");
            buzException.setStatus(401);
            throw buzException;
        }

        File file = new File(filePath);
        if (file.isDirectory()) {
            throw new BuzException("非法文件");
        }

        if (!file.exists()) {
            throw new BuzException("文件不存在");
        }

        try {
            FileCopyUtils.copy(new FileInputStream(file), destination);
        } catch (IOException e) {
            throw new BuzException("文件流写入失败");
        }
    }

    @Override
    public void deleteFile(String filePath) {
        if (StringUtils.isEmpty(filePath)) {
            throw new BuzException("文件名无效");
        }

        File file = new File(rootDirectory, filePath);
        if (file.isDirectory() || !file.exists()) {
            return;
        }

        if (!file.delete()) {
            throw new BuzException("文件存在，但删除失败");
        }

    }

    public boolean isOverride() {
        return override;
    }

    public void setOverride(boolean override) {
        this.override = override;
    }
}