package top.daxian1218.dust.edp.services.storage;

import com.mongodb.client.MongoDatabase;
import com.mongodb.client.gridfs.GridFSBucket;
import com.mongodb.client.gridfs.GridFSBuckets;
import com.mongodb.client.gridfs.GridFSFindIterable;
import com.mongodb.client.gridfs.GridFSUploadStream;
import com.mongodb.client.model.Filters;
import org.apache.commons.lang3.StringUtils;
import org.bson.conversions.Bson;
import org.bson.types.ObjectId;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import top.daxian1218.dust.commons.id.SnowFlakeIdWorker;
import top.daxian1218.dust.edp.BuzException;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * mongo仓库
 *
 * @author daxian
 */
public class MongoFileRepository implements IFileRepository {
    private final SnowFlakeIdWorker idWorker;
    Logger logger = LoggerFactory.getLogger(MongoFileRepository.class);
    private final MongoTemplate mongoTemplate;
    private Lock locker = new ReentrantLock();

    public MongoFileRepository(MongoTemplate mongoTemplate) {
        this.mongoTemplate = mongoTemplate;
        this.idWorker = new SnowFlakeIdWorker(1, 0);
    }

    private MongoDatabase getDatabase() {
        return mongoTemplate.getDb();
    }

    @Override
    public String uploadFile(String filePath, InputStream source) {
        if (source == null)
        {
            throw new BuzException("未找到数据流");
        }

        MongoDatabase database = getDatabase();
        GridFSBucket bucket = GridFSBuckets.create(database);
        GridFSUploadStream stream =  bucket.openUploadStream(filePath);
        byte[] buffer = new byte[1024];
        if (source.markSupported()) {
            try {
                source.reset();
            } catch (IOException e) {
                logger.error("重置上传流异常", e);
            }
        }

        while (true) {
            try {
                int len = source.read(buffer);
                if (len <= 0) {
                    break;
                }

                stream.write(buffer, 0 , len);
            } catch (IOException e) {
                logger.error("读取上传流异常", e);
            }
        }

        stream.flush();
        stream.close();
        return stream.getObjectId().toString();
    }

    @Override
    public void downloadFile(String filePath, OutputStream destination) {
        MongoDatabase database = getDatabase();
        GridFSBucket bucket = GridFSBuckets.create(database);
        bucket.downloadToStream(new ObjectId(filePath), destination);
    }

    @Override
    public void deleteFile(String filePath) {
        MongoDatabase database = getDatabase();
        GridFSBucket bucket = GridFSBuckets.create(database);
        bucket.delete(new ObjectId(filePath));
    }

    public boolean exists(String id)
    {
        MongoDatabase database = getDatabase();
        Bson filter = Filters.eq("_id", new ObjectId(id));
        GridFSBucket bucket = GridFSBuckets.create(database);
        GridFSFindIterable files = bucket.find(filter);
        return files.iterator().hasNext();
    }

    @Override
    public FormData upload(HttpServletRequest request) {
        if (request == null) {
            throw new BuzException("request is null");
        }
        MultipartHttpServletRequest multipartRequest = null;
        if (request instanceof MultipartHttpServletRequest) {
            multipartRequest = (MultipartHttpServletRequest) request;
        }

        List<FileInfo> files = multipartRequest == null
                ? uploadFileByStream(request)
                : uploadFileByFormData(multipartRequest);
        if (logger.isInfoEnabled()) {
            logger.info("files has upload:" + files.stream().map(FileInfo::toString).reduce(String::concat));
        }
        FormData formData = new FormData();
        files.forEach(f -> f.setFileId(f.getFilePath()));
        formData.setFiles(files);

        multipartRequest.getParameterMap().forEach((key, values) -> {
            formData.set(key, values != null && values.length > 0 ? values[0] : null);
        });
        return formData;
    }

    @Override
    public void download(String id, OutputStream os) {
        if (StringUtils.isEmpty(id)) {
            throw new BuzException("id is null");
        }

        downloadFile(id, os);
    }

    @Override
    public void delete(String id) {
        if (StringUtils.isEmpty(id)) {
            throw new BuzException("id is null");
        }
        deleteFile(id);
    }

    List<FileInfo> uploadFileByStream(HttpServletRequest request) {
        FileInfo fileInfo = new FileInfo();
        fileInfo.setFileId(String.valueOf(idWorker.nextId()));

        String fileName = request.getParameter("name");
        if (StringUtils.isEmpty(fileName)) {
            fileName = fileInfo.getFileId();
        }
        fileInfo.setFileName(fileName);

        try {
            String path = this.uploadFile(fileName, request.getInputStream());
            fileInfo.setFilePath(path);
        } catch (IOException e) {
            throw new BuzException("请求流读取失败");
        }

        return Collections.singletonList(fileInfo);
    }

    List<FileInfo> uploadFileByFormData(MultipartHttpServletRequest request) {
        List<FileInfo> files = new ArrayList<>();
        request.getFileMap().forEach((name, multiFile) -> {
            if (multiFile.getSize() > 0) {
                FileInfo file = new FileInfo();
                file.setFileId(String.valueOf(idWorker.nextId()));

                String fileName = multiFile.getOriginalFilename();
                if (StringUtils.isEmpty(fileName)) {
                    fileName = multiFile.getName();
                }

                if (StringUtils.isEmpty(fileName)) {
                    fileName = file.getFileId();
                }

                file.setFileName(fileName);
                try {
                    file.setFilePath(uploadFile(file.getFileName(), multiFile.getInputStream()));
                } catch (IOException e) {
                    throw new BuzException("请求流读取失败");
                }

                files.add(file);
            }
        });

        return files;
    }
}
