package top.daxian1218.dust.edp.services.storage;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import top.daxian1218.dust.commons.exception.DustSystemException;
import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.commons.id.SnowFlakeIdWorker;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.DictObjRow;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.services.BaseService;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author daxian
 */
@Service
public class StorageService extends BaseService {
    private final Logger _logger = LoggerFactory.getLogger(StorageService.class);
    private final LocalFileRepository localFileRepository;
    private MongoFileRepository mongoFileRepository;
    private final StorageOptions storageOptions;

    public StorageService(StorageOptions storageOptions,
                          LocalFileRepository localFileRepository) {
        this.storageOptions = storageOptions;
        this.localFileRepository = localFileRepository;
    }

    @Autowired(required = false)
    public void setMongoFileRepository( MongoFileRepository mongoFileRepository) {
        this.mongoFileRepository = mongoFileRepository;
    }

    private IFileRepository getRepository() {
        switch (storageOptions.getLocation()) {
            case MONGO:
                if (mongoFileRepository == null) {
                    throw new DustSystemException("mongo配置错误");
                }
                return mongoFileRepository;
            case LOCAL:
                return localFileRepository;
            default:
                throw new DustSystemException("未指定存储位置", "0005", null);
        }
    }

    private FormData upload(HttpServletRequest request) {
        IFileRepository repo = getRepository();
        return repo.upload(request);
    }

    public List<String> uploadFiles(HttpServletRequest request) {
        return upload(request).getFiles().stream().map(fileInfo -> fileInfo.getFileId()).collect(Collectors.toList());
    }


    public void download(String id, HttpServletResponse context) {
        OutputStream os;
        try {
            context.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + id + "\"");
            os = context.getOutputStream();
        } catch (IOException e) {
            throw new BuzException("请求无法写入");
        }

        IFileRepository repo = getRepository();
        repo.download(id, os);

        try {
            os.flush();
            os.close();
        } catch (IOException e) {
            _logger.error("下载文件flush失败", e);
        }
    }

    public void delete(String id) {
        IFileRepository repo = getRepository();
        repo.delete(id);
    }

    /**
     * 判断文件存储中，文件是否存在
     *
     * @param id
     */
    public boolean fileExists(String id) {
        IFileRepository repo = getRepository();
        return repo.exists(id);
    }
    public InputStream getFile(String fileId) {
        if (!fileExists((fileId))) {
            return null;
        }

        ByteArrayOutputStream ms = new ByteArrayOutputStream();
        try {
            IFileRepository repo = getRepository();
             repo.download(fileId, ms);

            return new ByteArrayInputStream(ms.toByteArray());
        } finally {
            try {
                ms.close();
            } catch (IOException e) {
                _logger.error("关闭文件流失败", e);
            }
        }
    }

    public String uploadFile(String fileName, InputStream inputStream) {
        IFileRepository repo = getRepository();
        return repo.uploadFile(fileName, inputStream);
    }
}