package top.daxian1218.dust.edp.services.sys;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import top.daxian1218.dust.db.DbConstants;
import top.daxian1218.dust.db.cache.ICache;
import top.daxian1218.dust.db.cache.ICacheManager;
import top.daxian1218.dust.db.dict.*;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.dict.condition.OperationType;
import top.daxian1218.dust.db.dict.condition.expressions.ColumnExpression;
import top.daxian1218.dust.db.dict.condition.expressions.ValueListExpression;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.edp.DictCodes;
import top.daxian1218.dust.edp.services.BaseService;
import top.daxian1218.dust.service.security.AuthoritiesConstants;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author daxian
 */
@Service
public class MenuService extends BaseService {

    private final ICache cache;
    public static final String ALL_MENU = "ALL_MENU";
    public static final String DEV_PERMS = "dev";
    private IMenuFilter filter;

    public MenuService(ICacheManager cacheManager) {
        this.cache = cacheManager.getCache(DbConstants.CACHE_CAFFEINE_LONG);
    }

    @Autowired
    public void setFilter(IMenuFilter filter) {
        this.filter = filter;
    }

    public List<MenuNode> ListMenu() {
        String[] roles = getRoles();
        if (roles.length == 0) {
            throw new BuzException("用户未配置角色");
        }

        // 获取所有菜单数据
        List<MenuNode> allMenus;
        if (!cache.exists(ALL_MENU)) {
            DictObj sysMenu = DictObjBuilder.create("SysMenuUI", AppCodes.SysApp);
            OrderBy orderByPid = new OrderBy();
            orderByPid.setColumnName("pid");
            orderByPid.setOrderByType(OrderByType.ASC);

            OrderBy orderByNum = new OrderBy();
            orderByNum.setColumnName("orderNum");
            orderByNum.setOrderByType(OrderByType.ASC);

            sysMenu.addOrderBy(orderByPid).addOrderBy(orderByNum);

            sysMenu.search();
            allMenus = BuilderTree(sysMenu.getRows());
            cache.put(ALL_MENU, JSON.toJSONString(allMenus));
        } else {
            allMenus = JSONArray.parseArray((String) cache.get(ALL_MENU), MenuNode.class);
        }

        // 获取当前用户所拥有的菜单权限
        List<String> ids = new ArrayList<>();
        boolean isSuperAdmin = Arrays.stream(getRoles()).anyMatch(r -> AuthoritiesConstants.ADMIN.equals(r));
        if (!isSuperAdmin) {
            DictObj objRoleMenu = DictObjBuilder.create(DictCodes.SysRoleMenu, AppCodes.SysApp);

            DictCondition roleCondition = new DictCondition();
            roleCondition.setOperation(OperationType.IN_LIST);
            roleCondition.setLeft(new ColumnExpression("roleCode"));
            roleCondition.setRight(new ValueListExpression(Arrays.asList(getRoles())));
            objRoleMenu.addCondition(roleCondition);
            objRoleMenu.search();
            if (objRoleMenu.getRows().size() == 0) {
                return new ArrayList<>();
            }

            for (DictObjRow row : objRoleMenu.getRows()) {
                ids.add(row.get("menuId").getStringValue());
            }
        }

        if (!isSuperAdmin) {
            RemoveInvalidMenu(allMenus, ids);

            if (filter != null) {
                allMenus = filter.doFilter(allMenus);
            }

        }

        return allMenus;
    }

    private void RemoveInvalidMenu(List<MenuNode> menus, List<String> ids) {
        String[] roles = getRoles();
        if (Arrays.stream(roles).allMatch(AuthoritiesConstants.ADMIN::equals)) {
            return;
        }

        for (int i = menus.size() - 1; i >= 0; i--) {
            MenuNode m = menus.get(i);
            List<MenuNode> children = m.getChildren();
            if (children != null && children.size() > 0) {
                RemoveInvalidMenu(children, ids);
                if (children.size() == 0) {
                    menus.remove(m);
                }
                continue;
            }

            // 修改逻辑，菜单组如果无子菜单则移除
            boolean isLeaf = m.isLeaf();
            String id = m.getMeta().getId();
            if (!isLeaf || !ids.contains(id)) {
                menus.remove(m);
            }
        }
    }


    private List<MenuNode> BuilderTree(List<DictObjRow> menus) {
        List<MenuNode> root = new ArrayList<>();
        for (DictObjRow m : menus) {
            if (StringUtils.isEmpty(m.get("path").getStringValue())) {
                continue;
            }

            if (0L == m.get("pid").getLongValue()) {
                root.add(ToMenuNode(m, FindTreeNodeChildren(menus, m.get("id").getLongValue())));
            }
        }
        return root;
    }

    private MenuNode ToMenuNode(DictObjRow row, List<MenuNode> children) {
        MenuNode node = new MenuNode();
        node.setId(row.get("id").getStringValue());
        node.setPath(row.get("path").getStringValue());
        node.setComponent(row.get("component").getStringValue());
        node.setName(row.get("code").getStringValue());
        node.setHidden(row.get("isHide").getBooleanValue());
        node.setParentId(row.get("pid").getStringValue());
        node.setOrder(row.get("orderNum").getInt32Value());

        String redirect = row.get("redirect").getStringValue();
        if (!StringUtils.isEmpty(redirect)) {
            node.setRedirect(redirect);
        }

        MenuNode.Meta meta = new MenuNode.Meta();
        meta.setTitle(row.get("title").getStringValue());
        meta.setIcon(row.get("icon").getStringValue());
        meta.setId(row.get("id").getStringValue());
        meta.setPerms(row.get("perms").getStringValue());
        if (StringUtils.equals(meta.getPerms(), DEV_PERMS)) {
            node.setHidden(false);
        }
        node.setMeta(meta);

        if (children != null && children.size() > 0) {
            node.getChildren().addAll(children);
        }

        node.setLeaf(node.getChildren().size() == 0);
        return node;
    }

    private List<MenuNode> FindTreeNodeChildren(List<DictObjRow> menus, long pId) {
        if (pId == 0) {
            return new ArrayList<>();
        }

        return menus.stream().filter(m -> pId == m.get("pid").getLongValue() && !StringUtils.isEmpty(m.get("path").getStringValue()))
                .map(m -> ToMenuNode(m, FindTreeNodeChildren(menus, m.get("id").getLongValue())))
                .collect(Collectors.toList());
    }

    /**
     * 加载父菜单的子菜单
     *
     * @param parentId
     * @return
     */
    public JSONArray LoadMenus(String parentId) {
        if (StringUtils.isEmpty(parentId)) {
            parentId = "0";
        }

        DictObj objMenu = new DictObj(DictCodes.SysMenu, AppCodes.DictApp);
        objMenu.addCondition(new DictCondition("pid", parentId));

        OrderBy orderBy = new OrderBy();
        orderBy.setColumnName("orderNum");
        orderBy.setOrderByType(OrderByType.ASC);
        objMenu.addOrderBy(orderBy);

        objMenu.search();
        JSONArray result = new JSONArray();
        objMenu.getRows().forEach(r ->
        {
            JSONObject jo = r.toJSON();
            result.add(jo);
        });
        return result;
    }

    public MenuNode getMenu(String id) {
        //TODO
        return null;
    }
}