package top.daxian1218.dust.edp.ui;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.commons.log.LoggerHelper;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.DictGlobalConfig;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.*;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.dict.condition.OperationType;
import top.daxian1218.dust.db.dict.condition.expressions.ColumnExpression;
import top.daxian1218.dust.db.dict.condition.expressions.CommandExpression;
import top.daxian1218.dust.db.dict.condition.expressions.ValueExpression;
import top.daxian1218.dust.db.dict.condition.expressions.ValueListExpression;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.AppCodes;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.service.security.SecurityUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class AdvancedQuery {
    private static final String DICT_QUERY_PROJECT = "QueryProject";
    private static final String DICT_PROJECT_ITEM = "QueryProjectItem";
    private static final String SUB_TABLE_ALIAS = "t";
    public static final String DEBUG_FLAG = "@@DUST";

    private final MenuStore menuStore;
    private final DictObj obj;
    private final IDataAdapter dataAdapter;

    private String projectId;
    private final List<QueryProjectItem> items = new ArrayList<>();

    public AdvancedQuery(MenuStore menuStore, DictObj obj, IDataAdapter dataAdapter) {
        Preconditions.checkNotNull(menuStore.getMenuId());
        Preconditions.checkNotNull(obj.getId());

        this.menuStore = menuStore;
        this.obj = obj;
        if (dataAdapter == null) {
            this.dataAdapter = DictGlobalConfig.Instance.getDataAdapter();
        } else {
            this.dataAdapter = dataAdapter;
        }
    }

    private void init() {
        DictObj objProject = new DictObj(DICT_QUERY_PROJECT, AppCodes.DictApp);
        objProject.addCondition(new DictCondition("menuId", menuStore.getMenuId()));
        objProject.addCondition(new DictCondition("objId", obj.getId()));

        String userId = SecurityUtils.getUserId();
        DictCondition type = new DictCondition("projectType", 1);
        type.setRequire(false);
        type.setOperation(OperationType.NOT_EQUAL);

        DictCondition where = new DictCondition();
        where.getSubConditions().add(new DictCondition("createBy", userId));
        where.getSubConditions().add(type);
        objProject.addCondition(where);

        objProject.search(dataAdapter);
        if (objProject.getRows().size() == 0) {
            return;
        }

        List<String> projectIds = objProject.getRows().stream()
                .map(r -> r.getStringValue("id")).collect(Collectors.toList());

        DictObj objItem = new DictObj(DICT_PROJECT_ITEM, AppCodes.DictApp);
        DictCondition condition = new DictCondition("projectId", projectId);
        condition.setOperation(OperationType.IN_LIST);
        condition.setLeft(new ColumnExpression("projectId"));
        condition.setRight(new ValueListExpression(projectIds));
        objItem.addCondition(condition);

        objItem.search(dataAdapter);
        objItem.getRows().forEach(r ->
        {
            ItemType itemType = ItemType.values()[r.getInt32Value("itemType")];
            if (itemType == ItemType.NONE) {
                return;
            }

            QueryProjectItem item = new QueryProjectItem();
            item.setColumn(r.getStringValue("itemName"));
            item.setItemType(itemType);
            item.setItemSrc(r.getLongValue("itemSrc"));
            item.setId(r.getStringValue("id"));
            items.add(item);
        });
    }

    private String getItemValue(Object v) {
        if (v instanceof Boolean) {
            boolean b = (boolean) v;
            return b ? "1" : "";
        }

        return TypeUtil.toString(v);
    }

    public void loadValueAsync(JSONObject json) {
        init();
        items.forEach(item ->
        {
            if (json.containsKey(item.getId())) {
                Object val = json.get(item.getId());
                if (item.getItemType() == ItemType.RANGE) {
                    if (val instanceof JSONArray) {
                        JSONArray ja = (JSONArray) val;
                        if (ja.size() > 0) {
                            item.setStart(getItemValue(ja.get(0)));

                            if (ja.size() > 1) {
                                item.setEnd(getItemValue(ja.get(1)));
                            }
                        }
                    } else {
                        item.setStart(getItemValue(val));
                    }
                    return;
                }

                item.setValue(getItemValue(val));
            }
        });
    }

    public DictCondition toCondition() {
        DictCondition condition = new DictCondition();
        if (items.size() == 0) {
            return condition;
        }

        for (QueryProjectItem item : items) {
            if (invalidItem(item)) {
                continue;
            }

            DictCondition itemCondition;
            if (item.getItemSrc() == 0L || item.getItemSrc() == this.obj.getId()) {
                itemCondition = getItemCondition(item, this.obj);
            } else {
                itemCondition = getDetailCondition(item);
            }

            if (itemCondition != null) {
                condition.getSubConditions().add(itemCondition);
            }
        }

        return condition;
    }

    private boolean invalidItem(QueryProjectItem item) {
        return StringUtils.isEmpty(item.getValue()) && StringUtils.isEmpty(item.getStart()) && StringUtils.isEmpty(item.getEnd());
    }

    private DictCondition getItemCondition(QueryProjectItem item, DictObj obj) {
        DictObjColumn objColumn = obj.getColumn(item.getColumn());
        if (objColumn == null) {
            LoggerHelper.debug("查询方案列不存在");
            return null;
        }

        if (StringUtils.equals(item.getValue(), DEBUG_FLAG)) {
            return new DictCondition("fakeColumn", DEBUG_FLAG);
        }

        DictCondition subCondition = new DictCondition();
        if (item.getItemType() == ItemType.RANGE) {
            if (!StringUtils.isEmpty(item.getStart())) {
                DictCondition d0 = new DictCondition();
                d0.setLeft(new ColumnExpression(item.getColumn()));
                if (objColumn.getDataType() == DataType.BOOLEAN) {
                    d0.setOperation(OperationType.EQUAL);
                    d0.setRight(new ValueExpression(item.getStart()).setDataType(objColumn.getDataType()));
                } else {
                    d0.setOperation(OperationType.EQUAL_GREATER);
                    d0.setRight(new ValueExpression(item.getStart()).setDataType(objColumn.getDataType()));
                }
                subCondition.getSubConditions().add(d0);
            }

            if (!StringUtils.isEmpty(item.getEnd()) && objColumn.getDataType() != DataType.BOOLEAN) {
                DictCondition d1 = new DictCondition();
                d1.setLeft(new ColumnExpression(item.getColumn()));
                d1.setOperation(OperationType.EQUAL_LESS);
                d1.setRight(new ValueExpression(item.getEnd()).setDataType(objColumn.getDataType()));
                subCondition.getSubConditions().add(d1);
            }

            return subCondition;
        }

        if (item.getItemType() == ItemType.MULTI_VALUES ) {
            String[] values = item.getValue().split(",");
            if (values.length > 1) {
                subCondition.setLeft(new ColumnExpression(item.getColumn()));
                subCondition.setOperation(OperationType.IN_LIST);
                subCondition.setRight(new ValueListExpression(Arrays.asList(values)));
                return subCondition;
            }

            subCondition.setLeft(new ColumnExpression(item.getColumn()));
            subCondition.setOperation(OperationType.CONTAIN);
            subCondition.setRight(new ValueExpression(item.getValue()).setDataType(objColumn.getDataType()));
            return subCondition;
        }

        subCondition.setLeft(new ColumnExpression(item.getColumn()));
        subCondition.setOperation(toOperation(item.getItemType(), objColumn));
        subCondition.setRight(new ValueExpression(item.getValue()).setDataType(objColumn.getDataType()));
        return subCondition;
    }

    private DictCondition getDetailCondition(QueryProjectItem item) {
        Optional<DictObjRelation> optionalRelation = obj.getRelations().stream().filter(r -> r.getDetail().getId() == item.getItemSrc()).findFirst();
        if (!optionalRelation.isPresent()) {
            return null;
        }

        DictObjRelation detailRelation = optionalRelation.get();
        DictObj detail = detailRelation.getDetail();
        detail.addCondition(getItemCondition(item, detail));
        SelectSqlGenerator ssg = new SelectSqlGenerator(detail);
        HSqlCommand cmd = ssg.getSubSql();

        DictObj master = detailRelation.getMaster();
        String masterTable = detailRelation.getMaster().getCode();
        String detailTable = detailRelation.getDetail().getCode();
        for (int i = 0; i < detailRelation.getMasterKeys().size(); i++) {
            DictObjColumn masterCol = master.getColumn(detailRelation.getMasterKeys().get(i));
            DictObjColumn detailCol = detail.getColumn(detailRelation.getDetailKeys().get(i));
            cmd.appendWhere(
                    String.format("%s.%s=%s.%s",
                            masterTable, masterCol.getPhysicalName(), detailTable, detailCol.getPhysicalName())
            );
        }
        return new DictCondition().setOperation(OperationType.EXIST).setRight(new CommandExpression(cmd));
    }

    private OperationType toOperation(ItemType itemType, DictObjColumn objColumn) {
        if (objColumn.getDataType() == DataType.BOOLEAN) {
            return OperationType.EQUAL;
        }

        switch (itemType) {
            case CONTAINS:
                return OperationType.CONTAIN;
            case EQUAL:
                return OperationType.EQUAL;
            case GREATER:
                return OperationType.EQUAL_GREATER;
            case LESS:
                return OperationType.EQUAL_LESS;
            case NOT_EQUAL:
                return OperationType.NOT_EQUAL;
            case START_WITH: {
                return OperationType.START_WITH;
            }
            case END_WITH: {
                return OperationType.END_WITH;
            }
            case MULTI_VALUES: {
                return OperationType.IN_LIST;
            }
        }

        throw new BuzException("itemType is not supported");
    }


    public enum ItemType {
        NONE, EQUAL, CONTAINS, GREATER, LESS, RANGE, NOT_EQUAL, START_WITH, END_WITH, MULTI_VALUES
    }

    public static class QueryProjectItem {
        private String id;
        private String column;
        private ItemType itemType;
        private String value;
        private String start;
        private String end;
        private Long itemSrc;

        public String getId() {
            return id;
        }

        public void setId(String id) {
            this.id = id;
        }

        public String getColumn() {
            return column;
        }

        public void setColumn(String column) {
            this.column = column;
        }

        public ItemType getItemType() {
            return itemType;
        }

        public void setItemType(ItemType itemType) {
            this.itemType = itemType;
        }

        public String getValue() {
            return value;
        }

        public void setValue(String value) {
            this.value = value;
        }

        public Long getItemSrc() {
            return itemSrc;
        }

        public void setItemSrc(Long itemSrc) {
            this.itemSrc = itemSrc;
        }

        public String getStart() {
            return start;
        }

        public void setStart(String start) {
            this.start = start;
        }

        public String getEnd() {
            return end;
        }

        public void setEnd(String end) {
            this.end = end;
        }
    }

}


