package top.daxian1218.dust.edp.ui;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.Error;
import top.daxian1218.dust.db.dict.*;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.edp.BuzException;
import top.daxian1218.dust.service.security.DustUser;
import top.daxian1218.dust.service.security.SecurityUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static top.daxian1218.dust.db.sql.SqlConstants.*;

/**
 * @author daxian
 */
public abstract class DictComponent implements IBiz {
    protected DustUser getCurrentUser() {
        return SecurityUtils.getUserInfo();
    }

    protected String[] getRoles() {
        if (StringUtils.isEmpty(getCurrentUser().getRoles())) {
            return new String[0];
        }

        return getCurrentUser().getRoles().split(",");
    }

    @Override
    public boolean BeforeSaveAsync(IDataAdapter adapter, MenuStore dict) {
        return true;
    }

    @Override
    public void ImportData(IDataAdapter adapter, MenuStore dict, List<DictObj> obj) {

    }

    @Override
    public void ExportData(IDataAdapter adapter, MenuStore dict, List<DictObj> obj) {

    }

    @Override
    public boolean BeforeDictObjSaveAsync(IDataAdapter adapter, MenuStore dict, DictObj obj) {
        return true;
    }

    private void SaveAsync(IDataAdapter adapter, MenuStore store, DictObj obj) {
        if (!obj.isReadonly() && obj.getChanges().size() > 0) {
            if (!BeforeDictObjSaveAsync(adapter, store, obj)) {
                return;
            }

            //优先删除操作，主表存在删除操作，优先删除明细
            if (obj.hasDeletedRow()) {
                List<DictObjRow> delRows = obj.getChanges().stream()
                        .filter(row -> row.getState() == RowState.DELETED)
                        .collect(Collectors.toList());

                bulkDeleteDetailAsync(adapter, obj, delRows);
            }

            boolean result = obj.save(adapter);
            if (!result) {
                String errMsg = obj.getErrors().stream()
                        .map(Error::getMessage)
                        .reduce(StringUtils.EMPTY, (s, error) -> s + StringUtils.LF + error);
                throw new BuzException(errMsg);
            }
            AfterDictObjSaveAsync(adapter, store, obj);
        }

        //主表保存后才会产生ID, 更新主从关系键，保存从表
        bindMasterDetail(adapter, store, obj, false);
        obj.getRelations().forEach(r ->
        {
            SaveAsync(adapter, store, r.getDetail());
        });

    }

    private void bulkDeleteDetailAsync(IDataAdapter adapter, DictObj obj, List<DictObjRow> masterRows) {
        if (obj.getRelations().size() == 0) {
            return;
        }
        obj.getRelations().forEach(relation -> {
            if (StringUtils.isEmpty(relation.getDetail().getTableName())) {
                return;
            }

            DictObj detail = relation.getDetail();
            String alias = detail.getCode();

            HSqlCommand delCmd = new HSqlCommand("DELETE FROM");
            delCmd.appendSql(detail.getTableName());
            delCmd.appendSql(alias);
            delCmd.appendSql(Where);

            for (int i = 0; i < relation.getDetailKeys().size(); i++) {
                String masterKey = relation.getMasterKeys().get(i);
                String detailPhysicalCol = detail.getColumn(relation.getDetailKeys().get(i)).getPhysicalName();
                if (i > 0) {
                    delCmd.appendSql(And);
                }
                delCmd.appendSql(alias + Dot + detailPhysicalCol + "=:" + masterKey);
            }

            masterRows.forEach(r ->
            {
                if (delCmd.hasParameter()) {
                    delCmd.applyBatch();
                }

                relation.getMasterKeys().forEach(k ->
                {
                    delCmd.setParameter(k, r.get(k).getValue());
                });
            });

            if (detail.getRelations().size() > 0) {
                bulkDeleteNestedDetailAsync(adapter, relation.getDetail(), delCmd);
            }

            adapter.update(delCmd);
        });
    }

    private void bulkDeleteNestedDetailAsync(IDataAdapter adapter, DictObj obj, HSqlCommand supCmd) {
        obj.getRelations().forEach(relation -> {
            if (StringUtils.isEmpty(relation.getDetail().getTableName())) {
                return;
            }

            DictObj detail = relation.getDetail();
            String alias = detail.getCode();
            HSqlCommand delCmd = new HSqlCommand("DELETE FROM " + detail.getTableName());
            delCmd.appendSql(alias);
            delCmd.appendSql(Where);
            delCmd.appendSql(Exists, false);
            delCmd.appendSql(LeftParen, false);
            delCmd.appendSql(delCmd.getCommandText().replace("DELETE", "SELECT 1"), false);
            delCmd.appendParameters(supCmd.getParameters());

            for (int i = 0; i < relation.getDetailKeys().size(); i++) {
                delCmd.appendSql(And);
                String masterKey = relation.getMasterKeys().get(i);
                String detailPhysicalCol = detail.getColumn(relation.getDetailKeys().get(i)).getPhysicalName();
                delCmd.appendSql(alias + Dot + detailPhysicalCol + "=" + obj.getCode() + Dot + masterKey);
            }

            delCmd.appendSql(RightParen, false);

            if (relation.getDetail().getRelations().size() > 0) {
                bulkDeleteNestedDetailAsync(adapter, relation.getDetail(), delCmd);
            }

            adapter.update(delCmd);
        });
    }

    @Override
    public List<DictObj> Save(IDataAdapter adapter, MenuStore store) {
        if (!BeforeSaveAsync(adapter, store)) {
            return new ArrayList<>();
        }

        store.getObjList().forEach(v -> {
            SaveAsync(adapter, store, v);
        });

        AfterSaveAsync(adapter, store);
        return store.getObjList();
    }

    @Override
    public void AfterDictObjSaveAsync(IDataAdapter adapter, MenuStore wrap, DictObj obj) {

    }

    @Override
    public void AfterSaveAsync(IDataAdapter adapter, MenuStore dict) {

    }

    @Override
    public boolean BeforeDictObjSearchAsync(IDataAdapter adapter, MenuStore menuStore, DictObj obj) {
        return true;
    }

    @Override
    public boolean BeforeSearchAsync(IDataAdapter adapter, MenuStore menuStore) {
        return true;
    }

    @Override
    public List<DictObj> SearchAsync(IDataAdapter adapter, MenuStore store) {
        if (!BeforeSearchAsync(adapter, store)) {
            return new ArrayList<>();
        }

        store.getObjList().forEach(obj -> {
            if (!obj.isReadonly()) {
                SearchAsync(adapter, store, obj);
            }


            // handle details search
            bindMasterDetail(adapter, store, obj, true);
            obj.getRelations().forEach(r -> {
                if (!r.getDetail().isReadonly()) {
                    SearchAsync(adapter, store, r.getDetail());
                }

            });
        });

        AfterSearchAsync(adapter, store);
        return store.getObjList();
    }

    protected void SearchAsync(IDataAdapter adapter, MenuStore store, DictObj obj) {
        if (!BeforeDictObjSearchAsync(adapter, store, obj)) {
            return;
        }

        if (obj.getRows().size() == 1) {
            DictObjRow row = obj.getRow(0);
            obj.getConditions().clear();
            obj.getColumns().forEach(col -> {
                if (col.isPrimaryKey()) {
                    obj.addCondition(col.getColumnName(), row.getValue(col.getColumnName()));
                }
            });

            obj.getPageInfo().setPageSize(1);
            obj.getPageInfo().setStart(0);
            obj.getPageInfo().setTotalRows(1);
        }

        HandleDataRightAsync(adapter, obj);
        obj.search(adapter);

        AfterDictObjSearchAsync(adapter, store, obj);
    }

    /**
     * 绑定主从关系，暂时只支持一主多从关系。
     */
    protected void bindMasterDetail(IDataAdapter adapter, MenuStore store, DictObj obj, Boolean query) {
        if (obj.getRelations().size() == 0) {
            return;
        }

        if (obj.getRows().size() == 1) {
            DictObjRow masterRow = obj.getFirstOrDefaultRow();
            obj.getRelations().forEach(md ->
            {
                if (masterRow == null) {
                    md.getDetail().setNoData(true);
                    return;
                }

                for (int i = 0; i < md.getDetailKeys().size(); i++) {
                    String detailKey = md.getDetailKeys().get(i);
                    String masterKey = md.getMasterKeys().get(i);
                    final Object masterValue = masterRow.get(masterKey).getValue();
                    if (query) {
                        md.getDetail().addCondition(new DictCondition(detailKey, masterValue));
                    } else {
                        md.getDetail().getRows().forEach(r -> {
                            r.get(detailKey).setValue(masterValue);
                        });
                    }
                }
            });
        }

        if (!query) {
            this.updateRelationKeys(adapter, store, obj);
        }
    }

    public void updateRelationKeys(IDataAdapter adapter, MenuStore store, DictObj obj) {
    }

    @Override
    public void AfterDictObjSearchAsync(IDataAdapter adapter, MenuStore menuStore, DictObj obj) {

    }

    @Override
    public void AfterSearchAsync(IDataAdapter adapter, MenuStore dict) {

    }

    @Override
    public void HandleDataRightAsync(IDataAdapter adapter, DictObj obj) {
    }

    @Override
    public DictCondition GetUserCondition(IDataAdapter adapter, DictObj obj, String columnName, String columnValue) {
        return null;
    }


    @Override
    public Object DoActionAsync(IDataAdapter adapter, String actionCode, MenuStore input) {
        return null;
    }

    protected String getSqlWhere(IDataAdapter adapter, DictObj dictObj) {
        return SqlBuilderFactory.getBuilder(adapter).getSelectWhere(dictObj);
    }

    /**
     * 获取固定条件的录入<br>
     * 假设字段为col, 则json的key映射为 <br>
     * <ul>
     *     <li>)col -> col大于<li/>
     *     <li>(col -> col小于<li/>
     *     <li>]col -> col大于等于<li/>
     *     <li>[col -> col小于等于<li/>
     *     <li>#col -> col等于null或者不等于null，取决于value是true（不等于null），还是false（等于null）<li/>
     *     <li>=col -> col等于<li/>
     *     <li>!col -> col不等于<li/>
     *     <li>%col -> col包含<li/>
     *     <li>^col -> col头部匹配<li/>
     *     <li>$col -> col尾部匹配<li/>
     *     <li>@col -> col在列表内，value以逗号隔开<li/>
     *
     * </ul>
     * @param dictObj
     * @return
     */
    protected JSONObject getConditionInput(DictObj dictObj) {
        return dictObj.getFixCondition() == null ? new JSONObject() : dictObj.getFixCondition().toFieldValues();
    }
}